'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var CoreApp = _interopDefault(require('fusion-core'));
var fusionReactAsync = require('fusion-react-async');
var server = require('react-dom/server');
var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

/* eslint-env node */
var serverRender = (el => `<div id='root'>${server.renderToString(el)}</div>`);

/* eslint-env browser */

var Provider = {
  create: name => {
    class Provider extends React.Component {
      getChildContext() {
        return {
          [name]: this.props.provides
        };
      }

      render() {
        return React.Children.only(this.props.children);
      }

    }

    Provider.childContextTypes = Object.assign({}, Provider.childContextTypes || {}, {
      [name]: PropTypes.object.isRequired
    });
    Provider.displayName = name.replace(/^./, c => c.toUpperCase()) + 'Provider';
    return Provider;
  }
};

var plugin = {
  create: (name, plugin, provider) => {
    let originalMiddleware = plugin.middleware;
    const ProviderComponent = provider || Provider.create(name);

    plugin.middleware = (deps, provides) => {
      if (originalMiddleware) {
        originalMiddleware = originalMiddleware(deps, provides);
      }

      return function (ctx, next) {
        if (ctx.element) {
          ctx.element = React.createElement(ProviderComponent, {
            provides,
            ctx
          }, ctx.element);
        }

        if (originalMiddleware) {
          return originalMiddleware(ctx, next);
        }

        return next();
      };
    };

    return plugin;
  }
};

var hoc = {
  create: (name, mapProvidesToProps) => {
    if (!mapProvidesToProps) {
      mapProvidesToProps = provides => ({
        [name]: provides
      });
    }

    return Component => {
      class HOC extends React.Component {
        constructor(props, ctx) {
          super(props, ctx);
          this.provides = ctx[name];
        }

        render() {
          const props = Object.assign({}, this.props, mapProvidesToProps(this.provides));
          return React.createElement(Component, props);
        }

      }

      const displayName = Component.displayName || Component.name;
      HOC.displayName = 'With' + name.replace(/^./, c => c.toUpperCase()) + '(' + displayName + ')';
      HOC.contextTypes = {
        [name]: PropTypes.object.isRequired
      };
      return HOC;
    };
  }
};

/* eslint-env browser */
class App extends CoreApp {
  constructor(root, render) {
    super(root, el => {
      return fusionReactAsync.prepare(el).then(() => {
        if (render) {
          return render(el);
        }

        return serverRender(el);
      });
    });
    this.middleware(fusionReactAsync.PreparePlugin());
  }

}

exports['default'] = App;
exports.ProviderPlugin = plugin;
exports.ProvidedHOC = hoc;
exports.Provider = Provider;
//# sourceMappingURL=index.js.map
